import { pools as _pools } from '../util/pools';
import { protectElement as _protectElement, } from '../util/memory';
import { components, upgrade } from '../element/custom';

var pools = _pools;
var protectElement = _protectElement;
var empty = {};

// Cache created nodes inside this object.
make.nodes = {};

/**
 * Converts a live node into a virtual node.
 *
 * @param node
 * @param protect
 * @return
 */
export default function make(node, protect) {
  // Default the nodeType to Element (1).
  let nodeType = 'nodeType' in node ? node.nodeType : 1;

  // Ignore attribute, comment, and CDATA nodes.
  if (nodeType === 2 || nodeType === 4 || nodeType === 8) {
    return false;
  }

  // Virtual representation of a node, containing only the data we wish to
  // diff and patch.
  let entry = pools.elementObject.get();

  entry.nodeName = node.nodeName.toLowerCase();

  // If the element is a text node set the nodeValue.
  if (nodeType === 3) {
    let nodeValue = node.textContent;
    entry.nodeValue = nodeValue;
  }

  entry.childNodes.length = 0;
  entry.attributes.length = 0;

  if (protect) {
    // Add to internal lookup.
    make.nodes[entry.uuid] = node;

    // Ensure this element will not be automatically cleaned up.
    protectElement(entry);
  }

  // Collect attributes.
  let attributes = node.attributes;

  // If the element has no attributes, skip over.
  if (attributes) {
    let attributesLength = attributes.length;

    if (attributesLength) {
      for (let i = 0; i < attributesLength; i++) {
        let attr = pools.attributeObject.get();

        if (protect) {
          pools.attributeObject.protect(attr);
        }

        attr.name = attributes[i].name;
        attr.value = attributes[i].value;

        entry.attributes[entry.attributes.length] = attr;
      }
    }
  }

  // Collect childNodes.
  let childNodes = node.childNodes ? node.childNodes : [];
  let childNodesLength = childNodes.length;

  // If the element has child nodes, convert them all to virtual nodes.
  if (node.nodeType !== 3 && childNodes) {
    for (let i = 0; i < childNodesLength; i++) {
      let newNode = make(childNodes[i], protect);

      if (newNode) {
        entry.childNodes[entry.childNodes.length] = newNode;
      }
    }
  }

  // TODO Rename this to first-run, because we're calling the attach callback
  // and protecting now.
  if (protect) {
    if (components[entry.nodeName]) {
      // Reset the prototype chain for this element. Upgrade will return `true`
      // if the element was upgraded for the first time. This is useful so we
      // don't end up in a loop when working with the same element.
      if (upgrade(entry.nodeName, node)) {
        // If the Node is in the DOM, trigger attached callback.
        if (node.parentNode && node.attachedCallback) {
          node.attachedCallback();
        }
      }
    }
  }

  return entry;
}
